#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__author__ = 'andy.hu'
__mtime__ = '2021/8/21'

"""
import os
import time
import random
import string
import asyncio

import boto3
import aiohttp
from aiohttp.client_exceptions import ServerTimeoutError
from qcloud_cos import CosConfig
from qcloud_cos import CosS3Client

from clife_svc.errors.error_code import ParameterException, UploadFileException, TimeoutException, DownloadFileException
from clife_svc.libs.log import klogger, clogger


class ClientRequest:
    _S3_CLIENT = ''
    HTTP_TIMEOUT = None
    COS_REGION = None
    COS_SECRET_ID = None
    COS_SECRET_KEY = None
    COS_BUCKET = None
    # COS_DIR = None
    COS_BUCKET_HOST = None
    # 请求超时时间
    __http_sess_timeout = None

    def __init__(self, app):
        self.HTTP_TIMEOUT = int(app.get_conf('http.timeout', 5))
        self.__http_sess_timeout = aiohttp.ClientTimeout(total=self.HTTP_TIMEOUT)

        self.S3_TYPE = None

        self.S3_ACCESS_KEY = app.get_conf('s3.access.key', '')
        self.S3_SECRET_KEY = app.get_conf('s3.secret.key', '')
        self.S3_BUCKET = app.get_conf('s3.bucket', '')
        self.S3_BUCKET_PRIVATE = int(app.get_conf('s3.bucket.private', '0'))
        self.S3_ENDPOINT_URL = app.get_conf('s3.endpoint.url', '')
        if not all([self.S3_ACCESS_KEY, self.S3_SECRET_KEY, self.S3_BUCKET, self.S3_ENDPOINT_URL]):
            clogger.warning('S3 Client missing required parameters, trying COS Client.')
            self.COS_REGION = app.get_conf('cos.region', '')
            self.COS_SECRET_ID = app.get_conf('cos.secret.id', '')
            self.COS_SECRET_KEY = app.get_conf('cos.secret.key', '')
            self.COS_BUCKET = app.get_conf('cos.bucket', '')
            self.COS_BUCKET_HOST = app.get_conf('cos.bucket.host', 'cos.clife.net')
            if not all([self.COS_REGION, self.COS_SECRET_ID, self.COS_SECRET_KEY, self.COS_BUCKET]):
                clogger.warning('COS Client missing required parameters, upload file may disabled.')
            else:
                self.S3_TYPE = 'COS'
                clogger.info('COS Client parameters check success')
        else:
            self.S3_TYPE = 'S3'
            clogger.info('S3 Client parameters check success')

        if self.S3_TYPE:
            clogger.info(f'Object Storage System: {self.S3_TYPE}')

    def create_txy_client(self):
        """
        腾讯云上传client对象
        """
        try:
            if not self.S3_TYPE:
                raise Exception
            elif self.S3_TYPE == 'COS':
                config = CosConfig(Region=self.COS_REGION, Secret_id=self.COS_SECRET_ID,
                                   Secret_key=self.COS_SECRET_KEY,
                                   Token=None)
                client = CosS3Client(config)
            else:
                client = boto3.client('s3',
                                      aws_access_key_id=self.S3_ACCESS_KEY,
                                      aws_secret_access_key=self.S3_SECRET_KEY,
                                      endpoint_url=self.S3_ENDPOINT_URL)
            return client
        except Exception:
            raise UploadFileException(data='create cos client error')

    @staticmethod
    async def _request_get(session, url, params, resp_type):
        async with session.get(url, params=params) as resp:
            if resp.status == 200:
                if resp_type == 'json':
                    return await resp.json()
                elif resp_type == 'text':
                    return await resp.text()
                else:
                    return await resp.read()
            else:
                klogger.error(
                    'Error of ClientRequest._request_get,resp.status:{},resp.text:{}'.format(resp.status,
                                                                                             await resp.text()))

    @staticmethod
    async def _request_post(session, url, data, json, resp_type):
        async with session.post(url, data=data, json=json) as resp:
            if resp.status == 200:
                if resp_type == 'json':
                    return await resp.json()
                elif resp_type == 'text':
                    return await resp.text()
                else:
                    return await resp.read()
            else:
                klogger.error(
                    'Error of ClientRequest._request_post,resp.status:{},resp.text:{}'.format(resp.status,
                                                                                              await resp.text()))

    async def _async_request(self, method, url, params=None, data=None, json=None, headers=None, cookies=None,
                             resp_type='json', timeout=None):
        """
        http请求
        :param method:
        :param url:
        :param params:
        :param data:
        :param headers:
        :param cookies:
        :param resp_type: json | text | byte
        :return:
        """
        if not timeout:
            timeout = self.__http_sess_timeout
        else:
            timeout = aiohttp.ClientTimeout(total=timeout)
        async with aiohttp.ClientSession(headers=headers, cookies=cookies, timeout=timeout) as sess:
            if method == 'GET':
                return await self._request_get(sess, url, params, resp_type=resp_type)
            elif method == 'POST':
                return await self._request_post(sess, url, data, json, resp_type=resp_type)
            else:
                raise ParameterException(data='async_request method must in [GET,POST]')

    async def download_file(self, file_url, retry=2, timeout=None):
        """
        图片下载，仅支持公有读权限的文件资源下载
        :param file_url:
        :param retry:
        :param timeout:
        :return: 文件字节数组
        """
        if 'http' in file_url:
            while True:
                klogger.info('Start download file: {}'.format(file_url))
                start = time.time()
                try:
                    resp_byte = await self._async_request('GET', file_url, resp_type='byte', timeout=timeout)
                    if not resp_byte:
                        raise DownloadFileException(data='Download file failed, url:{}'.format(file_url))
                    klogger.info('Download file cost: {}s'.format(round(time.time() - start, 2)))
                    klogger.info('Success download file.')
                    return resp_byte
                # 捕获超时类型异常进行重试
                except (ServerTimeoutError, asyncio.TimeoutError):
                    if retry != 0:
                        klogger.warning('Error download file,retry left: {}'.format(retry))
                        retry -= 1
                        continue
                    raise TimeoutException(data='Download file timeout, url:{}'.format(file_url))
                # 非超时异常直接抛出
                except:
                    raise DownloadFileException(data='Download file failed, url:{}'.format(file_url))
        else:
            # 本地文件路径格式直接返回
            klogger.info('File path: {}'.format(file_url))
            return ''

    @staticmethod
    def rename_file(file: str):
        """文件更名"""
        salt = ''.join(random.sample(string.ascii_letters + string.digits, 8))
        salt += str(int(time.time()))
        return salt + os.path.splitext(file)[1]

    async def upload_file_from_buffer(self, cos_dir: str, file_extension: str, body, retry=2) -> str:
        """
        :param cos_dir: 上传的cos子路径
        :param file_extension: 文件扩展名，如.txt|.png
        :param body: 文件流,必须实现了read方法
        :param retry: 失败重试次数
        :return: 文件url
        """
        if not self._S3_CLIENT:
            self._S3_CLIENT = self.create_txy_client()
        start = time.time()
        file_name = self.rename_file(str(time.time()) + file_extension)

        while retry >= 0:
            retry -= 1
            try:
                if self.S3_TYPE == 'COS':
                    cloud_path = '/' + cos_dir + '/' + file_name
                    client_resp = self._S3_CLIENT.upload_file_from_buffer(
                        Bucket=self.COS_BUCKET,
                        Key=cloud_path,
                        Body=body,
                        PartSize=10,
                        MAXThread=10
                    )
                    if client_resp and isinstance(client_resp, dict):
                        etag = str(client_resp.get('ETag', ''))
                        if etag:
                            file_url = 'http://' + self.COS_BUCKET_HOST + cloud_path
                            klogger.info('Upload file cost: {}s'.format(round(time.time() - start, 2)))
                            klogger.info('Upload file success: {}'.format(file_url))
                            return file_url
                    else:
                        klogger.warning('Error upload file,retry left: {}'.format(retry + 1))
                        continue
                else:
                    key = cos_dir + '/' + file_name
                    self._S3_CLIENT.upload_fileobj(
                        Fileobj=body,
                        Bucket=self.S3_BUCKET,
                        Key=key)  # noqa

                    file_url = self._S3_CLIENT.generate_presigned_url('get_object',
                                                                      Params={'Bucket': self.S3_BUCKET,
                                                                              'Key': key},
                                                                      ExpiresIn=600)
                    file_url = file_url if self.S3_BUCKET_PRIVATE else file_url.split('?')[0]
                    klogger.info('Upload file cost: {}s'.format(round(time.time() - start, 2)))
                    klogger.info('Upload file success: {}'.format(file_url))
                    return file_url
            except Exception:
                klogger.warning('Error upload file,retry left: {}'.format(retry + 1))
                continue
        raise UploadFileException

    async def upload_file(self, cos_dir: str, file_path: str, retry=2) -> str:
        """
        上传文件至腾讯云cos
        :param cos_dir 上传的cos子路径
        :param file_path 待上传的本地文件路径
        :param retry 失败重试次数
        :return 文件url
        """
        if not self._S3_CLIENT:
            self._S3_CLIENT = self.create_txy_client()

        if not os.path.isfile(file_path):
            raise ParameterException(data='file not exist:'.format(file_path))

        file_name = os.path.split(file_path)[1]
        file_name = self.rename_file(file_name)

        while retry > 0:
            retry -= 1
            start = time.time()

            try:
                if self.S3_TYPE == 'COS':
                    cloud_path = '/' + cos_dir + '/' + file_name
                    client_resp = self._S3_CLIENT.upload_file(
                        Bucket=self.COS_BUCKET,
                        LocalFilePath=file_path,
                        Key=cloud_path,
                        PartSize=10,
                        MAXThread=10
                    )
                    if client_resp and isinstance(client_resp, dict):
                        etag = str(client_resp.get('ETag', ''))
                        if etag:
                            file_url = 'http://' + self.COS_BUCKET_HOST + cloud_path
                            klogger.info('Upload file cost: {}s'.format(round(time.time() - start, 2)))
                            klogger.info('Upload file success: {}'.format(file_url))
                            return file_url
                    else:
                        klogger.warning('Error upload file,retry left: {}'.format(retry + 1))
                        continue
                else:
                    key = cos_dir + '/' + file_name
                    self._S3_CLIENT.upload_file(
                        Filename=file_path,
                        Bucket=self.S3_BUCKET,
                        Key=key)  # noqa

                    file_url = self._S3_CLIENT.generate_presigned_url('get_object',
                                                                      Params={'Bucket': self.S3_BUCKET,
                                                                              'Key': key},
                                                                      ExpiresIn=600)
                    file_url = file_url if self.S3_BUCKET_PRIVATE else file_url.split('?')[0]
                    klogger.info('Upload file cost: {}s'.format(round(time.time() - start, 2)))
                    klogger.info('Upload file success: {}'.format(file_url))
                    return file_url
            except Exception:
                klogger.warning('Error upload file,retry left: {}'.format(retry + 1))
                continue
        raise UploadFileException

    async def call_back(self, url: str, body: dict, retry=2, timeout=None):
        while retry > 0:
            retry -= 1
            klogger.info('Start call back: UUID:{}, URL:{}, Body: {}'.format(body.get('uuid'), url, body))
            start = time.time()
            try:
                resp = await self._async_request('POST', url, json=body, timeout=timeout)
                if resp:
                    klogger.info('Call back cost: {}s'.format(round(time.time() - start, 2)))
                    klogger.info('Success call back.')
                    return resp
            except (ServerTimeoutError, asyncio.TimeoutError):
                if retry > 0:
                    klogger.warning('Error call back,retry left: {}'.format(retry + 1))
                    continue
                raise TimeoutException(data='call back timeout')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__author__ = 'andy.hu'
__mtime__ = '2021/8/25'

"""
from typing import Optional, Dict, Any

from fastapi import HTTPException


class ApiException(HTTPException):
    status_code: int = 500  # 响应状态码
    error_code: int = 10000  # 错误状态码
    data: dict = {}
    msg: str = 'Server error'
    headers: Optional[Dict[str, Any]] = None

    def __init__(self,  data=None, headers=None):
        if data:
            self.data = data
        if headers:
            self.headers = headers

        super(ApiException, self).__init__(
            status_code=self.status_code, detail=self.msg, headers=self.headers
        )

    def __repr__(self) -> str:
        class_name = self.__class__.__name__
        return f"{class_name}(status_code={self.status_code!r},error_code={self.error_code!r}, msg={self.msg!r}),data={self.data!r}"


class ParameterException(ApiException):
    """
    参数异常
    error_code
    """
    status_code = 400
    error_code = 10001
    msg = 'Invalid parameter'


class TimeoutException(ApiException):
    """
    超时异常
    """
    status_code = 500
    error_code = 10002
    msg = 'Request timeout'


class DownloadFileException(ApiException):
    """
    文件下载异常
    error_code
    """
    status_code = 400
    error_code = 10003
    msg = 'File download failed'


class UploadFileException(ApiException):
    """
    文件上传异常
    """
    status_code = 400
    error_code = 10004
    msg = 'File upload failed'


class SoException(ApiException):
    """
    so库错误
    """
    status_code = 500
    error_code = 10005
    msg = 'So error'


class ContentException(ApiException):
    """
    内容错误
    """
    status_code = 400
    error_code = 10006
    msg = 'Content error'


class DecodeImageException(ApiException):
    """
    图片解码错误
    """
    status_code = 400
    error_code = 10007
    msg = 'Decode image error'


class EncodeImageException(ApiException):
    """
    图片编码错误
    """
    status_code = 500
    error_code = 10008
    msg = 'Encode image error'


class ProcessImageException(ApiException):
    """
    图片处理错误
    """
    status_code = 500
    error_code = 10009
    msg = 'Process image error'


class Base64DecodeException(ApiException):
    """
    Base64解码错误
    """
    status_code = 400
    error_code = 10010
    msg = 'Base64 decode error'


class UrlDecodeException(ApiException):
    """
    UrlDecode解码错误
    """
    status_code = 400
    error_code = 10011
    msg = 'URL decode error'


class ResourceException(ApiException):
    """
    GPU/显存资源不足，服务临时不可用
    """
    status_code = 500
    error_code = 10012
    msg = 'URL decode error'


class SliceImageException(ApiException):
    """
    图片分块错误
    """
    status_code = 500
    error_code = 10013
    msg = 'Slice image error'


class MergeImageException(ApiException):
    """
    图片块合并错误
    """
    status_code = 500
    error_code = 10014
    msg = 'Merge image error'


class CallBackException(ApiException):
    """
    回调异常
    """
    status_code = 500
    error_code = 10015
    msg = 'Call back error'


class UnknownException(ApiException):
    """
    未知异常
    """
    status_code = 500
    error_code = 10024
    msg = 'Unknown error'




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__author__ = 'YAnho.wen'
__mtime__ = '2021/12/21'

"""
import os
import datetime
import random
import hashlib

PREFIX = "CLIFE_"


def format_multi_value(context: str):
    """
    对多值内容进行字典转换
    """
    value_dict = {}
    lines = [_ for _ in context.replace('↵', '\n').splitlines() if not _.strip().startswith('#')]
    for line in lines:
        contents = line.replace(" ", "").split('=')
        if len(contents) == 2:
            value_dict[contents[0]] = contents[1]
    return value_dict


def get_env(name):
    """
    从环境变量中获取key对应的value
    """
    for env_key in [name, name.upper(),
                    "_".join([_ for _ in name.split('.')]),
                    "_".join([_.upper() for _ in name.split('.')])]:
        for key in [env_key, PREFIX + env_key]:
            value = os.environ.get(key)
            if value is not None:
                return value


def get_md5(string):
    hl = hashlib.md5()
    hl.update(string.encode(encoding='utf-8'))
    return hl.hexdigest()


def tid_maker():
    return '{0:%Y%m%d%H%M%S%f}'.format(datetime.datetime.now()) + ''.join([str(random.randint(0, 9)) for _ in range(5)])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__author__ = 'andy.hu'
__mtime__ = '2021/07/09'

"""
import os
import re
import time
import threading
import json
from typing import (
    Any,
    Callable,
    List,
    Optional,
    Set,
    Union,
)

from fastapi import FastAPI, APIRouter, Request, Response
from fastapi.middleware.cors import CORSMiddleware
from fastapi.responses import ORJSONResponse
from fastapi.routing import APIRoute
from starlette.middleware.base import BaseHTTPMiddleware, RequestResponseEndpoint
from starlette.requests import Request as HttpRequest
from starlette.responses import Response as HttpResponse

from clife_svc.libs import utils
from clife_svc.libs.context import request_id
from clife_svc.config.disconf import Disconf
from clife_svc.config.configmap import ConfigMap
from clife_svc.errors.error_code import ApiException
from clife_svc.libs.http_request import ClientRequest
from clife_svc.libs.log import init_log, klogger, tlogger
from clife_svc.libs.mq_handler import MQHandler


class AiServiceRoute(APIRoute):

    def get_route_handler(self) -> Callable:
        original_route_handler = super().get_route_handler()

        async def custom_route_handler(request: Request) -> Response:
            body = await request.body()
            if request.query_params:
                klogger.info('Request Params: {}'.format(request.query_params))
            if body:
                try:
                    klogger.info('Request Body: {}'.format(json.loads(body.decode('utf-8'))))
                except:
                    klogger.info('Request Body: {}'.format(body.decode('utf-8')))
            before = time.time()
            # 这里可以获取的我们的请求体的信息----
            response: Response = await original_route_handler(request)

            # 下面可以处理我们的响应体的报文信息，未被异常处理器拦截的请求将继续执行
            duration = time.time() - before
            if request.scope['path'] == '/time':
                tlogger.info('Request Cost: {}s'.format(round(duration, 2)))
                tlogger.info('Response Content: {}'.format(response.body.decode('utf-8')))
            else:
                klogger.info('Request Cost: {}s'.format(round(duration, 2)))
                klogger.info('Response Content: {}'.format(response.body.decode('utf-8')))
            return response

        return custom_route_handler


class App(object):
    """
    http接口服务上下文对象，单实例对象
    """
    _instance = None
    _instance_lock = threading.Lock()

    def __new__(cls, *args, **kwargs):
        """
        实现单例模式
        """
        if not cls._instance:
            with cls._instance_lock:
                if not cls._instance:
                    cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self, app_name: str, log_root_path: str = '/www/logs', *conf: Union[str, list]):
        """
        构造函数
        :param app_name 项目名称
        :param log_root_path 项目输出的日志根路径，推荐使用/www/logs，便于线上统一采集日志
        :param conf: 配置文件名称列表，提供字符串列表或逗号分隔字符串
        """

        # app_name参数校验

        if not re.match(r'^([a-zA-Z0-9]+-?[a-zA-Z0-9]+)+$', app_name):
            raise Exception('app_name can only be letters, numbers, or strike-through!')

        self.__app_name = app_name
        init_log(os.path.join(log_root_path, app_name), log_level='DEBUG')

        self.__disconf_item = Disconf('clife-ai', '0.0.1-SNAPSHOT', conf).get_disconf()
        self.__configmap = ConfigMap()

        self.__ClientRequest = ClientRequest(self)
        self.__MQHandler = MQHandler(app=self, app_name=app_name)

        self.__fast_api = FastAPI(title='ai-service', default_response_class=ORJSONResponse)
        self.__ai_router = APIRouter(route_class=AiServiceRoute)

    def init_api(self) -> FastAPI:
        """
        在App中初始化服务接口
        :return: FastAPI，作为服务器运行入口对象
        """
        self.__init_middlewares()
        self.__fast_api.add_exception_handler(ApiException, api_exception_handler)
        self.__fast_api.add_exception_handler(Exception, app_exception)
        self.__ai_router.add_api_route('/time', endpoint=index, methods=['GET'], name='time')
        self.__fast_api.include_router(self.__ai_router)
        self.__MQHandler.start_consumer()
        return self.__fast_api

    def __init_middlewares(self):
        self.__fast_api.add_middleware(CORSMiddleware,
                                       allow_credentials=True,
                                       allow_origins=["*"],
                                       allow_methods=["*"],
                                       allow_headers=["*"], )
        self.__fast_api.add_middleware(Interceptor)

    def get_conf(self, key: str, default: str = '') -> str:
        """
        获取disconf或ConfigMap挂载文件或环境变量的配置信息
        :param key:配置项的key
        :param default:配置项默认值
        :return:
        """
        if key in self.__disconf_item:
            return self.__disconf_item.get(key)
        item = self.__configmap.get(key)
        if item:
            return item
        item = utils.get_env(key)
        if item:
            return item
        return default

    def get_conf_list(self, key: str, default: list = None) -> list:
        """
        获取列表形式配置数据
        :param key:配置项的key
        :param default:配置项默认值
        :return:
        """
        if default is None:
            default = []
        values = self.get_conf(key)
        if values:
            return values.split(',')
        return default

    def get_all_conf(self) -> dict:
        """
        获取所有配置数据
        :return:
        """
        all_config = {}
        all_config.update(self.__disconf_item)
        all_config.update(self.__configmap.get_all())
        return all_config

    def add_api(self, path: str, endpoint: Callable[..., Any], methods: Optional[Union[Set[str], List[str]]] = None):
        """
        增加服务接口，此函数需要在init_api前调用
        :param path:接口访问路径
        :param endpoint:接口实现函数
        :param methods:接口访问方式，如GET、POST等
        :return:
        """
        self.__ai_router.add_api_route(path, endpoint, methods=methods)

    def add_subscribe(self, call_back, topic=None):
        """
        :param call_back:回调函数
        :param topic:订阅主题
        """
        self.__MQHandler.add_subscribe(call_back, topic)

    async def download_file(self, file_url, retry=2, timeout=None):
        """
        下载文件
        :param timeout:
        :param file_url:文件地址
        :param retry:失败重试次数，默认为2次，建议不大于3次
        :param timeout: 文件下载超时时间（秒），默认为配置文件ai-commons.properties中http.timeout，目前为15秒
        :return:文件数据字节数组
        """

        '''
        cos_cli = self.__ClientRequest.create_txy_client()
        buckets_list = cos_cli.list_buckets()
        for bucket in buckets_list['Buckets']['Bucket']:
            print(bucket)
            acl = cos_cli.get_bucket_acl(bucket['Name'])
            print(acl)
        '''
        return await self.__ClientRequest.download_file(file_url, retry, timeout)

    async def upload_file(self, file_path: str, retry=2) -> str:
        """
        :param file_path:本地文件路径
        :param retry:失败重试次数，默认为2次，建议不大于3次
        :return: 文件url
        """
        return await self.__ClientRequest.upload_file(self.__app_name, file_path, retry)

    async def upload_file_from_buffer(self, file_extension: str, body, retry=2) -> str:
        """
        :param file_extension: 文件扩展名，如.txt|.png
        :param body: 文件流,必须实现了read方法
        :param retry: 失败重试次数,默认为2次，建议不大于3次
        :return: 文件url
        """
        return await self.__ClientRequest.upload_file_from_buffer(self.__app_name, file_extension, body, retry)

    async def send_mq_msg(self, body, topic=None, keys=None, tags=None):
        """
        :param body: rocketMQ消息内容
        :param topic: rocketMQ消息主题，非必传，未配置默认主题时必传
        :param keys: rocketMQ消息唯一标识，非必传
        :param tags: rocketMQ消息标签，非必传
        """
        self.__MQHandler.send_sync(body, topic, keys, tags)

    async def call_back(self, url: str, body: dict, retry=2, timeout=None):
        """
        推理结果回调
        :param url:回调地址
        :param body:回调内容，字典中必须包含‘uuid’的key，且value符合UUID规则
        :param retry:失败重试次数，默认为2次，建议不大于3次
        :param timeout: 回调超时时间（秒），默认为配置文件ai-commons.properties中http.timeout，目前为15秒
        :return:回调请求响应体
        """
        if not re.match(r'[0-9a-f]{8}(-[0-9a-f]{4}){3}-[0-9a-f]{12}', body.get('uuid', '')):
            raise Exception('The request body of call back must contain the correct UUID.')
        return await self.__ClientRequest.call_back(url, body, retry, timeout)


class Interceptor(BaseHTTPMiddleware):
    """
    拦截所有请求
    """

    async def dispatch(self, request: HttpRequest, call_next: RequestResponseEndpoint) -> HttpResponse:
        # 生成请求标识
        request_id.set(utils.tid_maker())
        # 记录客户端请求的URL，包括未定义的URL，
        # 拦截器中不能获取request中body内容，会导致请求阻塞
        if request.scope['path'] == '/time':
            tlogger.info('Request URL: {} {}'.format(request.method, request.url))
            response = await call_next(request)
            tlogger.info('Response HTTP Status Code: {}'.format(response.status_code))
        else:
            klogger.info('Request URL: {} {}'.format(request.method, request.url))
            response = await call_next(request)
            klogger.info('Response HTTP Status Code: {}'.format(response.status_code))
        return response


async def index(q: Optional[str] = None):
    """k8s 探针 http监控服务请求地址"""
    result = {'code': 0,
              'msg': 'success',
              'data': {'time': time.strftime('%Y-%m-%d-%H-%M', time.localtime())}}
    if q:
        result['data'].update({'q': q})
    return result


async def api_exception_handler(request: Request, exc: ApiException) -> ORJSONResponse:
    """拦截接口抛出的所有自定义的HTTPException 异常"""
    klogger.exception('Request Exception:'.format())
    response = ORJSONResponse({
        "code": exc.error_code,
        "msg": exc.msg,
        "data": exc.data
    }, status_code=exc.status_code)
    klogger.info('Response Content:{}'.format(response.body.decode('utf-8')))
    klogger.info('Response HTTP Status Code: {}'.format(response.status_code))
    return response


async def app_exception(request: Request, exc: Exception) -> ORJSONResponse:
    """拦截接口抛出的所有未知非HTTPException 异常"""
    klogger.exception('Request Exception:'.format())
    response = ORJSONResponse({
        "code": 10024,
        "msg": 'Unknown error',
        "data": {},
    }, status_code=500)
    klogger.info('Response Content:{}'.format(response.body.decode('utf-8')))
    klogger.info('Response HTTP Status Code: {}'.format(response.status_code))
    return response



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
ConfigMap工具
__author__ = 'YAnho.wen'
__mtime__ = '2021/12/28'

"""
import os

from clife_svc.libs import utils
from clife_svc.libs.log import clogger


class ConfigMap:
    """
    从 ConfigMap 挂载文件获取配置
    """

    def __init__(self):
        self.__configmap_path = os.path.join(os.getcwd(), 'configmap')
        clogger.info('CONFIGMAP_PATH: {}'.format(self.__configmap_path))

        clogger.info('Start loading configmap files...')
        self.configmap = self._load_configmap_file()
        clogger.info('Load configmap files finished.')
        self._log_all_conf()

    def get(self, key):
        """
        获取ConfigMap文件的配置内容
        """
        return self.get_all().get(key)

    def get_all(self):
        """
        获取所有ConfigMap
        """
        config_map = self._load_configmap_file()
        if self.configmap != config_map:
            clogger.info('The ConfigMap has been updated.')
            self.configmap = config_map
            self._log_all_conf()
        return config_map

    def _log_all_conf(self):
        for key, value in self.configmap.items():
            if any([_ in key for _ in ['secret', 'secret'.upper()]]):
                clogger.info('configmap_item: {}=MD5({})'.format(key, utils.get_md5(value)))
            else:
                clogger.info('configmap_item: {}={}'.format(key, value))

    def _load_configmap_file(self):
        """
        获取最新的ConfigMap文件内容
        """
        config_map = {}
        if os.path.exists(self.__configmap_path):
            for file_name in os.listdir(self.__configmap_path):
                file_path = os.path.join(self.__configmap_path, file_name)
                if os.path.isfile(file_path):
                    with open(file_path, 'r', encoding='utf-8') as f:
                        items = utils.format_multi_value(f.read())
                    config_map.update(items)
        return config_map



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__author__ = 'YAnho.wen'
__mtime__ = '2021/12/21'

"""
from contextvars import ContextVar
from typing import Optional

# 申明请求ID变量
request_id: ContextVar[Optional[str]] = ContextVar('Id of request', default="")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__author__ = 'andy.hu'
__mtime__ = '2021/07/09'

"""
import os
import sys

from loguru import logger as klogger

from clife_svc.libs.context import request_id


def _console_log_filter(record) -> bool:
    record['req_id'] = request_id.get()
    return True


def _format(record):
    if record['req_id']:
        return "<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green> | " \
               "<level>{level: <8}</level> | " \
               "<cyan>{name}</cyan>:<cyan>{function}</cyan>:<cyan>{line}</cyan> | " \
               "<cyan>{req_id}</cyan> | " \
               "<level>{message}</level>\n{exception}"
    else:
        return "<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green> | " \
               "<level>{level: <8}</level> | " \
               "<cyan>{name}</cyan>:<cyan>{function}</cyan>:<cyan>{line}</cyan> | " \
               "<level>{message}</level>\n{exception}"


klogger.remove(handler_id=None)
klogger.add(sys.stderr, filter=_console_log_filter, format=_format)
tlogger = klogger.bind(time=True)
clogger = klogger.bind(config=True)


def init_log(log_path: str, log_level='INFO'):
    """
    初始化服务日志模块
    :param log_path: 日志输出路径
    :param log_level: 日志级别，从低到高依次为 TRACE|DEBUG|INFO|SUCCESS|WARNING|ERROR|CRITICAL
    :return:
    """
    # 配置项日志
    klogger.add(os.path.join(log_path, 'config.log'), format=_format,
                filter=lambda record: record['extra'].get('config'))

    # 探针日志
    klogger.add(os.path.join(log_path, 'probe.log'), format=_format, retention='1 week',
                filter=lambda record: record['extra'].get('time'))
    # 通用日志
    klogger.add(os.path.join(log_path, '{time:YYYY-MM-DD}.log'),
                format=_format,
                level=log_level,
                enqueue=True,
                rotation='00:00',
                retention='10 days',
                encoding='utf-8',
                compression='zip',
                filter=lambda record: not record['extra'].get('time') and
                                      not record['extra'].get('config'))


if __name__ == '__main__':
    print(os.path.dirname(__file__))
    print(os.path.dirname(os.path.dirname(__file__)))
    print(os.path.dirname(os.path.dirname(os.path.dirname(__file__))))
    print(os.path.abspath(os.path.dirname(os.path.dirname(os.path.dirname(__file__)))))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
配置中心
__author__ = 'andy.hu'
__mtime__ = '2021/07/09'

"""
import os
import sys

import requests

from clife_svc.libs.log import clogger
from clife_svc.libs import utils


_ENVIRONMENT = {
    0: {  # 开发环境
        'env_name': 'rd',
        'disconf_url': 'http://disconf.clife.net:8099/disconf-web/api'
    },
    1: {  # 测试环境
        'env_name': 'itest',
        'disconf_url': 'http://disconf.clife.net:8099/disconf-web/api'
    },
    2: {  # 生产环境
        'env_name': 'res',
        'disconf_url': 'http://disconf.clife.net:8099/disconf-web/api'
    },
    -1: {  # 本地环境
        'env_name': 'rd',
        'disconf_url': 'http://10.6.14.85:8099/disconf-web/api'
    },
}


class Disconf:
    """
    从 disconf 下载配置文件
    """

    def __init__(self, apps, version, keys: tuple):
        """ 构造函数
        :param apps: disconf配置app，多个使用逗号分隔
        :param version: 配置版本号
        :param keys: 配置文件名，缺省时只加载ai-commons.properties
        """
        __ENV = int(os.environ.get("ENVIRONMENT", -1))
        self.__ROOT_DIR = os.path.abspath(os.path.dirname(sys.argv[0]))

        self.__apps = self._split2list(apps)
        self.__sess = requests.session()
        self.__version = version
        self.__disconf_url = _ENVIRONMENT.get(__ENV).get('disconf_url')
        self.__env_name = _ENVIRONMENT.get(__ENV).get('env_name')
        # 下载disconf配置到根路径文件
        self.__disconf_path = os.path.join(self.__ROOT_DIR, 'local.properties')
        # 配置文件名称
        self.configs = ['ai-commons.properties', ] + self._split2list(keys)

        clogger.info('ROOT_PATH: {}'.format(self.__ROOT_DIR))
        clogger.info('ENVIRONMENT: {}'.format(__ENV))
        clogger.info('ENV_NAME: {}'.format(self.__env_name))
        clogger.info('DISCONF_URL: {}'.format(self.__disconf_url))

    @staticmethod
    def _split2list(keys):
        config_list = []
        if isinstance(keys, str):
            config_list.extend([_.strip() for _ in keys.replace('，', ',').split(',')])
        else:
            for key in keys:
                if isinstance(key, str):
                    config_list.extend([_.strip() for _ in key.replace('，', ',').split(',')])
                elif isinstance(key, list) or isinstance(key, tuple):
                    for _ in key:
                        config_list.append(_)
        return config_list

    def _download_file(self, app, key) -> dict:
        """
        从配置中心获取配置文件
        :param app: disconf配置app
        :param key: disconf配置文件的key（即配置文件名称）
        :return: 配置数据字典
        """

        url = self.__disconf_url + '/config/file'
        params = {
            'app': app,
            'version': self.__version,
            'env': self.__env_name,
            'key': key
        }
        try:
            resp = self.__sess.get(url, params=params, timeout=10)
            if resp.status_code == 200:
                try:
                    return utils.format_multi_value(resp.text)
                except Exception as e:
                    clogger.error('Error format value：{}'.format(e))
                    raise e
            else:
                clogger.warning('Get disconf "{}" error, status_code:{}, response:{}'.format(key,
                                                                                             resp.status_code,
                                                                                             resp.text))
        except Exception as e:
            clogger.error('Get disconf error:{}'.format(e))
            raise e

    def get_disconf(self) -> dict:
        """
        获取当前项目下的所有配置数据字典
        :return:
        """
        clogger.info('Start downloading disconf files...')
        disconf_item = {}
        try:
            for app_name in self.__apps:
                for conf_key in self.configs:
                    item = self._download_file(app_name, conf_key)
                    if item:
                        disconf_item.update(item)
            self.__sess.close()

            if not disconf_item:
                clogger.error('Error download disconf from {}.'.format(self.__disconf_url))
                if os.path.exists(self.__disconf_path):
                    clogger.info('Try to load local disconf...')
                    with open(self.__disconf_path, 'r') as f:
                        disconf_item = utils.format_multi_value(f.read())
                    if not disconf_item:
                        clogger.info('Faild to load local disconf.')
                    else:
                        clogger.info('Load local disconf success.')
            else:
                clogger.info('Download disconf files finished.')

            for key, value in disconf_item.items():
                if any([_ in key for _ in ['secret', 'secret'.upper()]]):
                    clogger.info('disconf_item: {}=MD5({})'.format(key, utils.get_md5(value)))
                else:
                    clogger.info('disconf_item: {}={}'.format(key, value))
        except Exception as e:
            clogger.error('Error download disconf:{}'.format(e))
        finally:
            return disconf_item



import setuptools

# Readme
with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

# Module dependencies
requirements, dependency_links = [], []
with open('clife_svc/requirements.txt') as f:
    for line in f.read().splitlines():
        if line.startswith('-e git+'):
            dependency_links.append(line.replace('-e ', ''))
        else:
            requirements.append(line)

setuptools.setup(
    name='clife_svc',   # 需要打包的名字,即本模块要发布的名字
    version='1.01',     # 版本
    author='andy.hu',  # 作者名
    author_email='hlp0@163.com',  # 作者邮件
    description='A module for service',   # 简要描述
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='',       # 项目地址,一般是代码托管的网站
    packages=setuptools.find_packages(),
    include_package_data=False,
    classifiers=[
        'Programming Language :: Python :: 3',
        "License :: OSI Approved :: MIT License",
        'Operating System :: OS Independent',
    ],
    install_requires=requirements,
    dependency_links=dependency_links,
    python_requires='>=3.6',
)



from clife_svc.libs.log import klogger, clogger


class MQHandler:

    def __init__(self, app, app_name):
        self._app = app
        self._producer = None
        self._producer_topic = None
        self._consumer = None
        self._consumer_group = None
        self._consumer_sub_topics = []
        try:
            from rocketmq.client import Producer, PushConsumer, Message
        except NotImplementedError:
            clogger.warning(f'rocketMQ does not support Windows, Message Queue disabled.')
        else:
            try:
                rocket_url = app.get_conf('rocket.mq.url')
                if rocket_url:
                    clogger.info(f'rocketMQ URL: {rocket_url}')
                    # 创建生产者
                    self._producer = Producer(app_name)
                    self._producer.set_namesrv_addr(rocket_url)
                    self._producer.start()
                    self._producer_topic = app.get_conf('rocket.mq.producer.topic')
                    if not self._producer_topic:
                        clogger.warning(f'rocketMQ Producer: Default topic not found. Topic required when send message.')
                    # 创建消费者
                    self._consumer_group = app.get_conf('rocket.mq.consumer.group')
                    if not self._consumer_group:
                        clogger.warning(f'rocketMQ Consumer: consumer group not found, consumer disabled.')
                    else:
                        self._consumer = PushConsumer(self._consumer_group)
                        self._consumer.set_namesrv_addr(rocket_url)
                else:
                    clogger.warning(f'rocketMQ URL not found, Message Queue disabled.')
            except Exception as e:
                clogger.warning(f'Error connect rocketMQ, Message Queue disabled.error_info:{e}')

    def add_subscribe(self, call_back, topic=None):
        if self._consumer:
            if not callable(call_back):
                raise Exception('Call back function must be callable')
            topic = topic if topic else self._app.get_conf('rocket.mq.consumer.topic')
            if not topic:
                raise Exception('Topic is required for rocketMQ Consumer')
            elif topic and topic in self._consumer_sub_topics:
                raise Exception(f'Consumer can not subscribe {topic} again')
            else:
                self._consumer_sub_topics.append(topic)
                self._consumer.subscribe(topic, call_back)
        else:
            raise Exception('rocketMQ consumer disabled')

    def start_consumer(self):
        if len(self._consumer_sub_topics) > 0:
            self._consumer.start()

    def send_sync(self, body, topic=None, keys=None, tags=None):
        from rocketmq.client import Message
        if self._producer:
            topic = topic if topic else self._producer_topic
            if topic:
                msg = Message(topic)  # noqa
                msg.set_body(body)
                if keys:
                    msg.set_keys(keys)
                if tags:
                    msg.set_tags(tags)
                self._producer.send_sync(msg)
                klogger.info('rocketMQ message send success')
            else:
                raise Exception('Topic is required for rocketMQ producer')
        else:
            raise Exception('rocketMQ producer disabled')
